#pragma once

#include "VectorLib.h"

#ifdef USE_MATRIX_INLINE
#define DECL inline
#else
#define DECL
#endif
class CMatrix33 
	{
	public:
	CVector3       Column[3];

	CMatrix33(void);
	CMatrix33(float V1);
	CMatrix33(const CMatrix33 &Matrix);
	CMatrix33(const CVector3 &V1, const CVector3 &V2, const CVector3 &V3);

	// Operadores internos
	DECL CVector3            &operator [] (unsigned int Index);
	DECL const CVector3      &operator [] (unsigned int Index) const;
	DECL CMatrix33           &operator =  (const CMatrix33 &Matrix);
	DECL CMatrix33           &operator += (const CMatrix33 &Matrix);
	DECL CMatrix33           &operator -= (const CMatrix33 &Matrix);
	DECL CMatrix33           &operator *= (const CMatrix33 &Matrix);
	DECL CMatrix33           &operator *= (float Float);

	// Operadores externos
	friend DECL bool         operator == (const CMatrix33 &V1, const CMatrix33 &V2);
	friend DECL bool         operator != (const CMatrix33 &V1, const CMatrix33 &V2);
	friend DECL CMatrix33    operator + (const CMatrix33 &V1, const CMatrix33 &V2);
	friend DECL CMatrix33    operator - (const CMatrix33 &V1, const CMatrix33 &V2);
	friend DECL CMatrix33    operator * (const CMatrix33 &V1, const CMatrix33 &V2);
	friend DECL CVector3     operator * (const CMatrix33 &Matrix, const CVector3 &V1);
	friend DECL CVector3     operator * (const CVector3 &V1, const CMatrix33 &Matrix);
	friend DECL CMatrix33    operator * (const CMatrix33 &Matrix, float Float);
	friend DECL CMatrix33    operator * (float Float, const CMatrix33 &Matrix);

	// Funcoes
	DECL CMatrix33           &Initialize(float Value);
	DECL CMatrix33           &Identity(void);
	DECL CMatrix33           &Transpose(void);
	DECL CMatrix33           &Invert(void);
	};

DECL CMatrix33    IdentityMatrix33();
DECL CMatrix33    TransposeMatrix33(const CMatrix33 &Matrix);
DECL CMatrix33    InvertMatrix33(const CMatrix33 &Matrix);
DECL CMatrix33    RotateRadMatrix33(float Radians);
DECL CMatrix33    TranslateMatrix33(float X, float Y);
DECL CMatrix33    ScaleMatrix33(float X, float Y, float Z = 1.0);
