#pragma once

#include "VectorLib.h"
#include "Matrix33.h"

#ifdef USE_MATRIX_INLINE
#define DECL inline
#else
#define DECL
#endif

class CMatrix44 
	{
	public:
	CVector4                 Column[4];

	CMatrix44(void);
	CMatrix44(float Value);
	CMatrix44(const CMatrix44 &Other);
	CMatrix44(const CVector4 &V0,const CVector4 &V1,const CVector4 &V2, const CVector4 &V3);
	explicit CMatrix44(const CMatrix33 &Other);

	// Operadores internos
	DECL CVector4            &operator [] (unsigned int Index);
	DECL const CVector4      &operator [] (unsigned int Index) const;
	DECL CMatrix44           &operator =  (const CMatrix44 &Other);
	DECL CMatrix44           &operator =  (const CMatrix33 &Other);
	DECL CMatrix44           &operator += (const CMatrix44 &Other);
	DECL CMatrix44           &operator -= (const CMatrix44 &Other);
	DECL CMatrix44           &operator *= (const CMatrix44 &Other);
	DECL CMatrix44           &operator *= (float Float);
	DECL CMatrix44           &operator /= (float Float);

	// Operadores externos
	friend DECL bool         operator == (const CMatrix44 &V1, const CMatrix44 &V2);
	friend DECL bool         operator != (const CMatrix44 &V1, const CMatrix44 &V2);
	friend DECL CMatrix44    operator + (const CMatrix44 &V1, const CMatrix44 &V2);
	friend DECL CMatrix44    operator - (const CMatrix44 &V1, const CMatrix44 &V2);
	friend DECL CMatrix44    operator * (const CMatrix44 &V1, const CMatrix44 &V2);
	friend DECL CVector3     operator * (const CMatrix44 &Other, const CVector3 &V1);
	friend DECL CVector3     operator * (const CVector3 &V1, const CMatrix44 &Other);
	friend DECL CVector4     operator * (const CMatrix44 &Other, const CVector4 &V1);
	friend DECL CVector4     operator * (const CVector4 &V1, const CMatrix44 &Other);
	friend DECL CMatrix44    operator * (const CMatrix44 &Other, float Float);
	friend DECL CMatrix44    operator * (float Float, const CMatrix44 &Other);

	DECL CMatrix44           &Identity(void);
	DECL CMatrix44           &Transpose(void);
	DECL CMatrix44           &Invert(void);
	};

#include "MatrixFunc.h"
