#include "MatrixFunc.h"

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

static inline float DegToRad(float Angle)
	{
	return Angle*0.01745329252f;
	};

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

static inline float RadToDeg(float Angle)
	{
	return Angle*57.29577951f;
	};

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 IdentityMatrix44(void)
	{
	CMatrix44 Return;
	return Return.Identity();
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 TransposeMatrix44(const CMatrix44 &Matrix)
	{
	CMatrix44 Return(Matrix);
	return Return.Transpose();
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 InvertMatrix44(const CMatrix44 &Matrix) 
	{
	CMatrix44 Return(Matrix);
	return Return.Invert();
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 RotateRadMatrix44(char Axis, float Radian) 
	{
	CMatrix44 Return;
	float sinA, cosA;

	sinA = (float)sin(Radian);
	cosA = (float)cos(Radian);

	switch (Axis)
		{
		case 'X':
			{
			Return[0][0] =  1.0F; Return[1][0] =  0.0F; Return[2][0] =  0.0F;
			Return[0][1] =  0.0F; Return[1][1] =  cosA; Return[2][1] = -sinA;
			Return[0][2] =  0.0F; Return[1][2] =  sinA; Return[2][2] =  cosA;
			break;
			}
		case 'Y':
			{
			Return[0][0] =  cosA; Return[1][0] =  0.0F; Return[2][0] =  sinA;
			Return[0][1] =  0.0F; Return[1][1] =  1.0F; Return[2][1] =  0.0F;
			Return[0][2] = -sinA; Return[1][2] =  0.0F; Return[2][2] =  cosA;
			break;
			}
		case 'Z':
			{
			Return[0][0] =  cosA; Return[1][0] = -sinA; Return[2][0] =  0.0F;
			Return[0][1] =  sinA; Return[1][1] =  cosA; Return[2][1] =  0.0F;
			Return[0][2] =  0.0F; Return[1][2] =  0.0F; Return[2][2] =  1.0F;
			break;
			}
		}

	Return[0][3] = 0.0F; Return[1][3] = 0.0F; Return[2][3] = 0.0F;
	Return[3][0] = 0.0F;
	Return[3][1] = 0.0F;
	Return[3][2] = 0.0F;
	Return[3][3] = 1.0F;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 RotateRadMatrix44(const CVector3 &Axis, float Radian) 
	{
	CMatrix44 Return;
	float sinA, cosA;
	float invCosA;
	CVector3 nrm = Axis;
	float X, Y, Z;
	float xSq, ySq, zSq;

	nrm.Normalize();
	sinA = (float)sin(Radian);
	cosA = (float)cos(Radian);
	invCosA = 1.0F - cosA;

	X = nrm.X;
	Y = nrm.Y;
	Z = nrm.Z;

	xSq = X * X;
	ySq = Y * Y;
	zSq = Z * Z;

	Return[0][0] = (invCosA * xSq) + (cosA);
	Return[1][0] = (invCosA * X * Y) - (sinA * Z );
	Return[2][0] = (invCosA * X * Z) + (sinA * Y );
	Return[3][0] = 0.0F;

	Return[0][1] = (invCosA * X * Y) + (sinA * Z);
	Return[1][1] = (invCosA * ySq) + (cosA);
	Return[2][1] = (invCosA * Y * Z) - (sinA * X);
	Return[3][1] = 0.0F;

	Return[0][2] = (invCosA * X * Z) - (sinA * Y);
	Return[1][2] = (invCosA * Y * Z) + (sinA * X);
	Return[2][2] = (invCosA * zSq) + (cosA);
	Return[3][2] = 0.0F;

	Return[0][3] = 0.0F;
	Return[1][3] = 0.0F;
	Return[2][3] = 0.0F;
	Return[3][3] = 1.0F;

	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 TranslateMatrix44(float X, float Y, float Z) 
	{
	CMatrix44 Return;

	Return.Identity();
	Return[3][0] = X;
	Return[3][1] = Y;
	Return[3][2] = Z;

	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 ScaleMatrix44(float X, float Y, float Z, float W) 
	{
	CMatrix44 Return;

	Return.Identity();
	Return[0][0] = X;
	Return[1][1] = Y;
	Return[2][2] = Z;
	Return[3][3] = W;

	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 LookAtMatrix44(const CVector3 &CameraPos,const CVector3 &Target,const CVector3 &CameraUp)
	{
	CMatrix44 Return;

	CVector3 F = Target - CameraPos;
	F.Normalize();

	CVector3 S = CrossProduct(F, Normalized(CameraUp));
	S.Normalize();

	CVector3 U = CrossProduct(S, F);
	U.Normalize();

	Return[0][0] = S.X;
	Return[1][0] = S.Y;
	Return[2][0] = S.Z;
	Return[3][0] = 0.0;

	Return[0][1] = U.X;
	Return[1][1] = U.Y;
	Return[2][1] = U.Z;
	Return[3][1] = 0.0;

	Return[0][2] = -F.X;
	Return[1][2] = -F.Y;
	Return[2][2] = -F.Z;
	Return[3][2] = 0.0;

	Return[0][3] = 0.0F;
	Return[1][3] = 0.0F;
	Return[2][3] = 0.0F;
	Return[3][3] = 1.0F;

	Return *= TranslateMatrix44(-CameraPos.X, -CameraPos.Y, -CameraPos.Z);

	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CMatrix44 FrustumMatrix44(float Left,float Right,float Top,float Bottom,float Near,float Far)
	{
	CMatrix44 Return;
	float width = Right-Left;
	float height = Top-Bottom;
	float depth = Far-Near;

	Return[0][0] = (2*Near) / width;
	Return[0][1] = 0.0F;
	Return[0][2] = 0.0F;
	Return[0][3] = 0.0F;

	Return[1][0] = 0.0F;
	Return[1][1] = (2*Near) / height;
	Return[1][2] = 0.0F;
	Return[1][3] = 0.0F;

	Return[2][0] = (Right + Left) / width;
	Return[2][1] = (Top + Bottom) / height;
	Return[2][2] = -(Far + Near) / depth;
	Return[2][3] = -1.0F;

	Return[3][0] = 0.0F;
	Return[3][1] = 0.0F;
	Return[3][2] = -(2*Far*Near) / depth;
	Return[3][3] = 0.0F;

	return Return;
	}

// Return V1 perspective CMatrix44 given the field-of-view in the Y
//   direction in degrees, the aspect ratio of Y/X, and near and
//   far plane distances.
CMatrix44 PerspectiveMatrix44(float fovY, float aspect, float Near, float Far) 
	{
	CMatrix44 Return;
	float angle;
	float cot;

	angle = fovY / 2.0F;
	angle = DegToRad( angle );

	cot = (float) cos(angle) / (float) sin(angle);

	Return[0][0] = cot / aspect;
	Return[0][1] = 0.0F;
	Return[0][2] = 0.0F;
	Return[0][3] = 0.0F;

	Return[1][0] = 0.0F;
	Return[1][1] = cot;
	Return[1][2] = 0.0F;
	Return[1][3] = 0.0F;

	Return[2][0] = 0.0F;
	Return[2][1] = 0.0F;
	Return[2][2] = -(Far + Near) / (Far - Near);
	Return[2][3] = -1.0F;


	Return[3][0] = 0.0F;
	Return[3][1] = 0.0F;
	Return[3][2] = -(2*Far*Near) / (Far - Near);
	Return[3][3] = 0.0F;

	return Return;
	}

// Return an orthographic CMatrix44 given the left, right, bottom, top,
//   near, and far values for the frustum boundaries.
CMatrix44 OrthoMatrix44(float Left, float Right,float Bottom, float Top, float Near, float Far)
	{
	CMatrix44 Return;
	float width = Right-Left;
	float height = Top-Bottom;
	float depth = Far-Near;

	Return[0][0] = 2.0F / width;
	Return[0][1] = 0.0F;
	Return[0][2] = 0.0F;
	Return[0][3] = 0.0F;

	Return[1][0] = 0.0F;
	Return[1][1] = 2.0F / height;
	Return[1][2] = 0.0F;
	Return[1][3] = 0.0F;

	Return[2][0] = 0.0F;
	Return[2][1] = 0.0F;
	Return[2][2] = -(2.0F) / depth;
	Return[2][3] = 0.0F;

	Return[3][0] = -(Right + Left) / width;
	Return[1][3] = -(Top + Bottom) / height;
	Return[3][2] = -(Far + Near) / depth;
	Return[3][3] = 1.0F;

	return Return;
	}

// Return an orientation matrix using 3 basis normalized CVectors
CMatrix44    OrthoNormalMatrix44(const CVector3 &xdir,const CVector3 &ydir, const CVector3 &zdir)
	{
	CMatrix44 Return;

	Return[0] = (CVector4)xdir;
	Return[1] = (CVector4)ydir;
	Return[2] = (CVector4)zdir;
	Return[3].Set(0.0, 0.0, 0.0, 1.0);

	return Return;
	}