#include "Quaternion.h"

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion::CQuaternion(const float NewX,const float NewY,const float NewZ,const float NewW)
	{
	SetValues(X,Y,Z,W);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion::CQuaternion(CVector3 Axis,float Angle)
	{
	float Temp1,Temp2;
	Temp1 = Axis.Length();
	assert(Temp1 != 0 && "Quaternion Axis is zero!");
	Temp2 = sinf(Angle * 0.5F) / Temp1;
	SetValues(Axis.X * Temp2,Axis.Y * Temp2,Axis.Z * Temp2,cosf(Angle * 0.5F));
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator () (const float NewX,const float NewY,const float NewZ)
	{
	SetValues(NewX,NewY,NewZ,W);
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator () (const float NewX,const float NewY,const float NewZ,const float NewW)
	{
	SetValues(NewX,NewY,NewZ,NewW);
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator () (CQuaternion &Other)
	{
	SetValues(Other.X,Other.Y,Other.Z,Other.W);
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator () (CVector3 Axis,float Angle)
	{
	float Temp1,Temp2;
	Temp1 = Axis.Length();
	assert(Temp1 != 0 && "Quaternion Axis is zero!");
	Temp2 = sinf(Angle * 0.5F) / Temp1;
	SetValues(Axis.X * Temp2,Axis.Y * Temp2,Axis.Z * Temp2,cosf(Angle * 0.5F));
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator = (CQuaternion &Other)
	{
	SetValues(Other.X,Other.Y,Other.Z,Other.W);
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator ~ ()
	{
	X=-X;
	Y=-Y;
	Z=-Z;
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::SetValues(float NewX,float NewY,float NewZ,float NewW)
	{
	X=NewX;
	Y=NewY;
	Z=NewZ;
	W=NewW;
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

bool CQuaternion::operator == (CQuaternion &Other)
	{
	if ((X==Other.X) && (Y==Other.Y) && (Z==Other.Z) && (W==Other.W)) return true;
	return false;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

bool CQuaternion::operator != (CQuaternion &Other)
	{
	if ((X!=Other.X) || (Y!=Other.Y) || (Z!=Other.Z) || (W!=Other.W)) return true;
	return false;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion CQuaternion::operator - ()
	{
	return CQuaternion(-X,-Y,-Z,-W);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion CQuaternion::operator + (CQuaternion &Other)
	{
	CQuaternion Temp;
	Temp(Other.X+X,Other.Y+Y,Other.Z+Z,Other.W+W);
	return Temp;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion CQuaternion::operator - (CQuaternion &Other)
	{
	CQuaternion Temp;
	Temp(Other.X-X,Other.Y-Y,Other.Z-Z,Other.W-W);
	return Temp;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion CQuaternion::operator * (CQuaternion &Other)
	{
	return CQuaternion(
		W * Other.X + X * Other.W + Y * Other.Z - Z * Other.Y,
		W * Other.Y + Y * Other.W + Z * Other.X - X * Other.Z,
		W * Other.Z + Z * Other.W + X * Other.Y - Y * Other.X,
		W * Other.W - X * Other.X - Y * Other.Y - Z * Other.Z); 
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator += (CQuaternion &Other)
	{
	X+=Other.X;
	Y+=Other.Y;
	Z+=Other.Z;
	W+=Other.W;
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator -= (CQuaternion &Other)
	{
	X-=Other.X;
	Y-=Other.Y;
	Z-=Other.Z;
	W-=Other.W;
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator *= (CQuaternion &Other)
	{
	X=W * Other.X + X * Other.W + Y * Other.Z - Z * Other.Y,
	Y=W * Other.Y + Y * Other.W + Z * Other.X - X * Other.Z,
	Z=W * Other.Z + Z * Other.W + X * Other.Y - Y * Other.X,
	W=W * Other.W - X * Other.X - Y * Other.Y - Z * Other.Z; 
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator /= (float &Scalar)
	{
	return SetValues(X/Scalar,Y/Scalar,Z/Scalar,W/Scalar);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::operator *= (float &Scalar)
	{
	return SetValues(X*Scalar,Y*Scalar,Z*Scalar,W*Scalar);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion &CQuaternion::SetEuler(float Yaw, float Pitch, float Roll)
	{
	float cosY = cosf(Yaw / 2.0F);
	float sinY = sinf(Yaw / 2.0F);
	float cosP = cosf(Pitch / 2.0F);
	float sinP = sinf(Pitch / 2.0F);
	float cosR = cosf(Roll / 2.0F);
	float sinR = sinf(Roll / 2.0F);
	SetValues(
		cosR * sinP * cosY + sinR * cosP * sinY,
		cosR * cosP * sinY - sinR * sinP * cosY,
		sinR * cosP * cosY - cosR * sinP * sinY,
		cosR * cosP * cosY + sinR * sinP * sinY
		);
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
	
CQuaternion &CQuaternion::Normalize(void)
	{
	float Length=GetLength();
	X/=Length;
	Y/=Length;
	Z/=Length;
	W/=Length;
	return *this;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

float CQuaternion::GetLength(void)
	{
	return sqrtf(W * W + X * X + Y * Y + Z * Z);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CMatrix33 CQuaternion::GetMatrix33(void)
	{
	return CMatrix33(
		CVector3(1 - 2*(Y*Y) - 2*(Z*Z), 2*(X*Y+W*Z), 2*(X*Z-W*Y)),
		CVector3(2*(X*Y-W*Z), 1 - 2*(X*X) - 2*(Z*Z), 2*(Y*Z+W*X)),
		CVector3(2*(X*Z+W*Y), 2*(Y*Z-W*X), 1 - 2*(X*X) - 2*(Y*Y)));
	}

//---------------------------------------------------------------------------
// Operadores externos
//---------------------------------------------------------------------------

CQuaternion operator * (const CQuaternion &First, const CQuaternion &Second)
	{
	return CQuaternion(
			First.W * Second.X + First.X * Second.W + First.Y * Second.Z - First.Z * Second.Y,
			First.W * Second.Y + First.Y * Second.W + First.Z * Second.X - First.X * Second.Z,
			First.W * Second.Z + First.Z * Second.W + First.X * Second.Y - First.Y * Second.X,
			First.W * Second.W - First.X * Second.X - First.Y * Second.Y - First.Z * Second.Z); 
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion operator + (const CQuaternion &First, const CQuaternion &Second)
	{
	return CQuaternion(First.X+Second.X,First.Y+Second.Y,First.Z+Second.Z,First.W+Second.W);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion operator - (const CQuaternion &First, const CQuaternion &Second)
	{
	return CQuaternion(First.X-Second.X,First.Y-Second.Y,First.Z-Second.Z,First.W-Second.W);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion operator - (const CQuaternion &Quaternion)
	{
	return CQuaternion(-Quaternion.X, -Quaternion.Y, -Quaternion.Z, -Quaternion.W);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion operator * (CQuaternion Quaternion,float Scalar)
	{
	return CQuaternion(Quaternion.X*Scalar,Quaternion.Y*Scalar,Quaternion.Z*Scalar,Quaternion.W*Scalar);
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion operator * (float Scalar,CQuaternion Quaternion)
	{
	return CQuaternion(Quaternion.X*Scalar,Quaternion.Y*Scalar,Quaternion.Z*Scalar,Quaternion.W*Scalar);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CQuaternion RotationArc(CVector3 V0,CVector3 V1)
	{
	CQuaternion Quaternion;
	// v0.normalize(); 
	// v1.normalize();  // If vector is already unit length then why do it again?
	CVector3 Temp = CrossProduct(V0,V1);

	float   d = DotProduct(V0,V1);
	float   s = (float)sqrt((1+d)*2);
	Quaternion.X = Temp.X / s;
	Quaternion.Y = Temp.Y / s;
	Quaternion.Z = Temp.Z / s;
	Quaternion.W = s /2.0f;
	return Quaternion;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

float DotProduct(const CQuaternion &First, const CQuaternion &Second)
	{
	return First.X * Second.X + First.Y * Second.Y + First.Z * Second.Z + First.W * Second.W;
	}

//---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------

CQuaternion Slerp(const CQuaternion &From, const CQuaternion &To, float Interpolation)
	{
	CQuaternion Temp;
	float omega, cosO, sinO;
	float scale0, scale1;

	cosO = DotProduct(From, To);

	if (cosO < 0.0)
		{
		cosO = -cosO;
		Temp = -To;
		}
	else
		{
		Temp = -To;
		}

	if ((1.0 - cosO) > ERROR_TOLERANCE)
		{
		omega = (float)acos(cosO);
		sinO = sinf(omega);
		scale0 = sinf((1.0F - Interpolation) * omega) / sinO;
		scale1 = sinf(Interpolation * omega) / sinO;
		}
	else
		{
		scale0 = 1.0F - Interpolation;
		scale1 = Interpolation;
		}
	return From*scale0 + Temp*scale1 ;
	}