#pragma once
#include "MatrixLib.h"

#define ERROR_TOLERANCE 1e-6

class CQuaternion
	{
	public:
	float X,Y,Z,W;

//---------------------------------------------------------------------------
// Constructores
//---------------------------------------------------------------------------

	CQuaternion(void) : X(0),Y(0),Z(0),W(1) {}
	CQuaternion(const float NewX,const float NewY,const float NewZ,const float NewW);
	CQuaternion(CVector3 Axis,float Angle);

//---------------------------------------------------------------------------
// Afectaes
//---------------------------------------------------------------------------

	CQuaternion              &operator () (const float NewX,const float NewY,const float NewZ);
	CQuaternion              &operator () (const float NewX,const float NewY,const float NewZ,const float NewW);
	CQuaternion              &operator () (CQuaternion &Other);
	CQuaternion              &operator () (CVector3 Axis,float Angle);
	CQuaternion              &operator = (CQuaternion &Other);// Afectao
	CQuaternion              &operator ~ ();// Conjugado
	CQuaternion              &SetValues(float NewX,float NewY,float NewZ,float NewW);

//---------------------------------------------------------------------------
// Comparaes
//---------------------------------------------------------------------------

	bool                     operator == (CQuaternion &Other);
	bool                     operator != (CQuaternion &Other);

//---------------------------------------------------------------------------
// Operaes com quaternions sem afectao
//---------------------------------------------------------------------------

	CQuaternion              operator - ();// Negao
	CQuaternion              operator + (CQuaternion &Other);// Adio
	CQuaternion              operator - (CQuaternion &Other);// Subtraco
	CQuaternion              operator * (CQuaternion &Other);// Multiplicao

//---------------------------------------------------------------------------
// Operaes com quaternions com afectao
//---------------------------------------------------------------------------

	CQuaternion              &operator += (CQuaternion &Other);// Soma com afectao
	CQuaternion              &operator -= (CQuaternion &Other);// Subtraco com afectao
	CQuaternion              &operator *= (CQuaternion &Other);// Multiplicao com afectao

//---------------------------------------------------------------------------
// Operaes com escalares com afectao
//---------------------------------------------------------------------------

	CQuaternion              &operator /= (float &Scalar);// Diviso com afectao
	CQuaternion              &operator *= (float &Scalar);// Multiplicao com afectao

//---------------------------------------------------------------------------
// Outras operaes
//---------------------------------------------------------------------------

	CQuaternion              &SetEuler(float Yaw, float Pitch, float Roll);// Transformao para angulo de Euler
	CQuaternion              &Normalize(void);// Normalizao dos vectores
	float                    GetLength (void);
	CMatrix33                GetMatrix33(void);
	};

//---------------------------------------------------------------------------
// Operadores externos
//---------------------------------------------------------------------------

CQuaternion operator * (const CQuaternion &First, const CQuaternion &Second);
CQuaternion operator + (const CQuaternion &First, const CQuaternion &Second);
CQuaternion operator - (const CQuaternion &First, const CQuaternion &Second);
CQuaternion operator - (const CQuaternion &Quaternion);

//---------------------------------------------------------------------------
// Multiplicao de quaternions por escalares
//---------------------------------------------------------------------------

CQuaternion operator * (CQuaternion Quaternion,float Scalar);
CQuaternion operator * (float Scalar,CQuaternion Quaternion);

//---------------------------------------------------------------------------
// Funes
//---------------------------------------------------------------------------

CQuaternion RotationArc(CVector3 V0,CVector3 V1);
inline float DotProduct(const CQuaternion &First, const CQuaternion &Second);
inline CQuaternion Slerp(const CQuaternion &From, const CQuaternion &To, float Interpolation);
