#include "Vector3.h"

//---------------------------------------------------------------------------
//  Operadores Internos
//---------------------------------------------------------------------------

float &CVector3::operator [] (unsigned int Index)
	{
	assert (Index<3);
	return *(&X+Index);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

const float &CVector3::operator [] (unsigned int Index) const
	{
	assert (Index<3);
	return *(&X+Index);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 &CVector3::operator =  (const CVector2 &V1)
	{
	X = V1.X;
	Y = V1.Y;
	Z = 0.0F;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 &CVector3::operator += (const CVector3 &V1)
	{
	X += V1.X;
	Y += V1.Y;
	Z += V1.Z;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 &CVector3::operator -= (const CVector3 &V1)
	{
	X -= V1.X;
	Y -= V1.Y;
	Z -= V1.Z;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 &CVector3::operator *= (float Scalar)
	{
	X *= Scalar;
	Y *= Scalar;
	Z *= Scalar;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 &CVector3::operator /= (float Scalar)
	{
    X /= Scalar;
    Y /= Scalar;
    Z /= Scalar;
    return *this;
	}

//---------------------------------------------------------------------------
//  Operadores internos
//---------------------------------------------------------------------------

bool CVector3::operator == (const CVector3 &V1)
	{
	return((X == V1.X) && (Y == V1.Y) && (Z == V1.Z));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool CVector3::operator != (const CVector3 &V1)
	{
	return((X != V1.X) || (Y != V1.Y) || (Z != V1.Z));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 CVector3::operator - (void)
	{
	return CVector3(-X, -Y, -Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 CVector3::operator + (const CVector3 &V1)
	{
	CVector3 Return(*this);
	Return += V1;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 CVector3::operator - (const CVector3 &V1)
	{
	CVector3 Return(*this);
	Return -= V1;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 CVector3::operator * (float Scalar)
	{
	return CVector3(Scalar * X, Scalar * Y, Scalar * Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 CVector3::operator / (float Scalar)
	{
	return CVector3(X / Scalar, Y / Scalar, Z / Scalar);
	}

//---------------------------------------------------------------------------
//  Operadores externos
//---------------------------------------------------------------------------

bool operator == (const CVector3 &V1,const CVector3 &V2)
	{
	return((V1.X == V2.X) && (V1.Y == V2.Y) && (V1.Z == V2.Z));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool operator != (const CVector3 &V1,const CVector3 &V2)
	{
	return((V1.X != V2.X) || (V1.Y != V2.Y) || (V1.Z != V2.Z));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 operator - (const CVector3 &V1)
	{
	return CVector3(-V1.X, -V1.Y, -V1.Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 operator + (const CVector3 &V1,const CVector3 &V2)
	{
	CVector3 Return(V1);
	Return += V2;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 operator - (const CVector3 &V1,const CVector3 &V2)
	{
	CVector3 Return(V1);
	Return -= V2;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 operator * (const CVector3 &V1, float Value)
	{
	return CVector3(Value * V1.X, Value * V1.Y, Value * V1.Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 operator * (float Value, const CVector3 &V1)
	{
	return CVector3(Value * V1.X, Value * V1.Y, Value * V1.Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 operator / (const CVector3 &V1, float Value)
	{
	return CVector3(V1.X / Value, V1.Y / Value, V1.Z / Value);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

void CVector3::Set(float NewX, float NewY, float NewZ)
	{
    X = NewX;
    Y = NewY;
    Z = NewZ;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

float CVector3::Length(void) const
	{
	return(float) sqrt(X*X + Y*Y + Z*Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

float CVector3::LengthSqr(void) const
	{
	return(X*X + Y*Y + Z*Z);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool CVector3::IsZero(void) const
	{
	return((X == 0.0F) && (Y == 0.0F) && (Z == 0.0F));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector3 &CVector3::Normalize(void)
	{
	float temp = Length();
	if (temp > 0.0F)
		temp = 1.0F / temp;
	else
		temp = 0.0F;
	X *= temp;
	Y *= temp;
	Z *= temp;
	return *this;
	}
