#include "Vector4.h"

//---------------------------------------------------------------------------
//  Operadores internos
//---------------------------------------------------------------------------

float &CVector4::operator [] (unsigned int Index)
	{
	assert (Index<4);
	return *(&X+Index);
	//return(Index == 0) ? X : (Index == 1) ? Y : (Index == 2) ? Z : W;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

const float &CVector4::operator [] (unsigned int Index) const
	{
	assert (Index<4);
	return *(&X+Index);
	//return(Index == 0) ? X : (Index == 1) ? Y : (Index == 2) ? Z : W;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::operator =  (const CVector3 &V1)
	{ 
	X = V1.X;
	Y = V1.Y;
	Z = V1.Z;
	W = 0.0F;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::operator =  (const CVector2 &V1)
	{
	X = V1.X;
	Y = V1.Y;
	Z = 0.0F;
	W = 0.0F;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::operator += (const CVector4 &V1)
	{
	X += V1.X;
	Y += V1.Y;
	Z += V1.Z;
	W += V1.W;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::operator -= (const CVector4 &V1)
	{
	X -= V1.X;
	Y -= V1.Y;
	Z -= V1.Z;
	W -= V1.W;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::operator *= (float Value)
	{
	X *= Value;
	Y *= Value;
	Z *= Value;
	W *= Value;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::operator /= (float Value)
	{
	X /= Value;
	Y /= Value;
	Z /= Value;
	W /= Value;
	return *this;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool CVector4::operator == (const CVector4 &Other)
	{
	return((X == Other.X) && (Y == Other.Y) &&
        (Z == Other.Z) && (W == Other.W));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool CVector4::operator != (const CVector4 &Other)
	{
	return((X != Other.X) || (Y != Other.Y) ||
        (Z != Other.Z) || (W != Other.W));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 CVector4::operator - (void)
	{
	return CVector4(-X, -Y, -Z, -W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 CVector4::operator + (const CVector4 &Other)
	{
	CVector4 Return(*this);
	Return += Other;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 CVector4::operator - (const CVector4 &Other)
	{
	CVector4 Return(*this);
	Return -= Other;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 CVector4::operator * (float Value)
	{
	return CVector4(Value * X, Value * Y, Value * Z, Value * W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 CVector4::operator / (float Value)
	{
    return CVector4(X / Value, Y / Value, Z / Value, W / Value);
	}

//---------------------------------------------------------------------------
//  Operadores externos
//---------------------------------------------------------------------------

bool operator == (const CVector4 &V1, const CVector4 &V2)
	{
	return((V1.X == V2.X) && (V1.Y == V2.Y) &&
        (V1.Z == V2.Z) && (V1.W == V2.W));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool operator != (const CVector4 &V1, const CVector4 &V2)
	{
	return((V1.X != V2.X) || (V1.Y != V2.Y) ||
        (V1.Z != V2.Z) || (V1.W != V2.W));
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 operator - (const CVector4 &V1)
	{
	return CVector4(-V1.X, -V1.Y, -V1.Z, -V1.W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 operator + (const CVector4 &V1, const CVector4 &V2)
	{
	CVector4 Return(V1);
	Return += V2;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 operator - (const CVector4 &V1, const CVector4 &V2)
	{
	CVector4 Return(V1);
	Return -= V2;
	return Return;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 operator * (const CVector4 &V1, float Value)
	{
	return CVector4(Value * V1.X, Value * V1.Y, Value * V1.Z, Value * V1.W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 operator * (float Value, const CVector4 &V1)
	{
	return CVector4(Value * V1.X, Value * V1.Y, Value * V1.Z, Value * V1.W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 operator / (const CVector4 &V1, float Value)
	{
    return CVector4(V1.X / Value, V1.Y / Value, V1.Z / Value, V1.W / Value);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

void CVector4::Set(float NewX, float NewY, float NewZ, float NewW)
	{
	X = NewX;
	Y = NewY;
	Z = NewZ;
	W = NewW;
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

float CVector4::Length(void) const
	{
	return(float) sqrt(X*X + Y*Y + Z*Z + W*W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

float CVector4::LengthSqr(void) const
	{
	return(X*X + Y*Y + Z*Z + W*W);
	}

//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

bool CVector4::IsZero(void) const
	{
	return((X == 0.0F) && (Y == 0.0F) && (Z == 0.0F) && (W == 0.0F));
	}


//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------

CVector4 &CVector4::Normalize(void)
	{
	float m = Length();
	if (m > 0.0F)
		m = 1.0F / m;
	else
		m = 0.0F;
	X *= m;
	Y *= m;
	Z *= m;
	W *= m;
	return *this;
	}
