#pragma once
#include <cmath>
#include <cassert>

#include "Vector2.h"
#include "Vector3.h"

class CVector4 
	{
	public:
	float X, Y, Z, W;

	CVector4(void): X(0), Y(0), Z(0), W(0) {};
	CVector4(float NewX, float NewY, float NewZ, float inW): X(NewX), Y(NewY), Z(NewZ), W(inW) {};
	CVector4(const CVector4 &V1): X(V1.X), Y(V1.Y), Z(V1.Z), W(V1.W) {};
	explicit CVector4(const CVector3 &V1): X(V1.X), Y(V1.Y), Z(V1.Z), W(0.0F) {};
	explicit CVector4(const CVector2 &V1): X(V1.X), Y(V1.Y), Z(0.0F), W(0.0F) {};

	// Operadores internos
	float                    &operator [] (unsigned int Index);
	const float              &operator [] (unsigned int Index) const;
	CVector4                 &operator =  (const CVector3 &V1);
	CVector4                 &operator =  (const CVector2 &V1);
	CVector4                 &operator += (const CVector4 &V1);
	CVector4                 &operator -= (const CVector4 &V1);
	CVector4                 &operator *= (float Value);
	CVector4                 &operator /= (float Value);

	bool                     operator == (const CVector4 &Other);
	bool                     operator != (const CVector4 &Other);
	CVector4                 operator - (void);
	CVector4                 operator + (const CVector4 &Other);
	CVector4                 operator - (const CVector4 &Other);
	CVector4                 operator * (float Value);
	CVector4                 operator / (float Value);

	// Operadores externos
	friend bool              operator == (const CVector4 &V1, const CVector4 &V2);
	friend bool              operator != (const CVector4 &V1, const CVector4 &V2);
	friend CVector4          operator - (const CVector4 &V1);
	friend CVector4          operator + (const CVector4 &V1, const CVector4 &V2);
	friend CVector4          operator - (const CVector4 &V1, const CVector4 &V2);
	friend CVector4          operator * (const CVector4 &V1, float f);
	friend CVector4          operator * (float f, const CVector4 &V1);
	friend CVector4          operator / (const CVector4 &V1, float f);

	// Funcoes
	void                     Set(float NewX, float NewY, float NewZ, float NewW);
	float                    Length(void) const;
	float                    LengthSqr(void) const;
	bool                     IsZero(void) const;
	CVector4                 &Normalize(void);
	};
